{% load admin_style_tags i18n %}
{% load static from staticfiles %}
<div id="header-logo">
    <img alt="Leprikón" src="{% static 'leprikon/img/logo-horizontal.png' %}" height="45"
         title="{% trans 'Version' %}: {{ request.leprikon_version }}"/>
</div>
<h1>{{ site_header|default:_('Django Administration') }}</h1>
<ul id="site-name" class="toolbar-item toolbar-item-navigation">
    <li>
        <a class="menu-item" href="">{{ site_header|default:_('Django Administration') }}</a>
        <ul class="submenu">
            {% if user.is_active and user.is_staff %}
                {% url 'django-admindocs-docroot' as docsroot %}
                {% if docsroot %}
                    <li>
                        <a href="{{ docsroot }}">{% trans 'Documentation' %}</a>
                    </li>
                    <li class="toolbar-item-navigation-break">-----</li>
                {% endif %}
            {% endif %}
            {% if user.has_usable_password %}
                <li>
                    <a href="{% url 'admin:password_change' %}">{% trans 'Change password' %}</a>
                </li>
                <li class="toolbar-item-navigation-break">-----</li>
            {% endif %}
            <li>
                <a href="{% url 'admin:logout' %}">
                    <span>{% trans 'Log out' %} {% firstof user.get_short_name user.get_username %}</span>
                </a>
            </li>
        </ul>
    </li>
</ul>
<div id="header-btn">
    <a href="/" class="icon-arrow-right btn js-header-link">
        {% trans 'Open' %} {% current_site_name %}
    </a>
</div>
